import('model.Component')
import('model.toolbar.header.ToolbarHeader')
import('model.toolbar.ToolbarContent')

---@class Toolbar : Component
Toolbar = Component:extendAs("model.toolbar.Toolbar")

function Toolbar:create(parent)
    local root = Toolbar:createRoot(parent, {
        type = "frame",
        direction = "vertical",
        style = "toolbar" })
    ToolbarHeader:create(root)
    ToolbarContent:create(root)
    return Toolbar:new(root)
end

function Toolbar:new(root)
    return Toolbar:super(Component:new(root))
end

function Toolbar:childrenClasses()
    return { ToolbarHeader, ToolbarContent }
end

---@return ToolbarHeader
function Toolbar:header()
    return self:child(ToolbarHeader)
end

---@return ToolbarContent
function Toolbar:content()
    return self:child(ToolbarContent)
end

---@public
function Toolbar:centerOnScreen()
    self:root().force_auto_center()
end

function Toolbar:handleGuiEvent(guiEvent)
    if Toolbar:isOriginOf(guiEvent) then
        if guiEvent:isLocationChange() then
            self:keepWithinTheScreen()
            self:turnOffCenteringOnScreen()
        end
    else
        Component.handleGuiEvent(self, guiEvent)
    end
end

---@private
function Toolbar:keepWithinTheScreen()
    local location = self:location()
    if location.x < 0 then
        location.x = 0
    end
    if location.y < 0 then
        location.y = 0
    end
    self:setLocation(location)
end

---@private
function Toolbar:turnOffCenteringOnScreen()
    self:root().auto_center = false
end

---@private
---@param location GuiLocation
function Toolbar:setLocation(location)
    self:root().location = location
end

---@private
---@return GuiLocation
function Toolbar:location()
    return self:root().location
end

---@public
function Toolbar:collapse()
    self:content():hide()
end

---@public
function Toolbar:expand()
    self:content():show()
end

---@public
function Toolbar:addSection()
    self:content():sectionList():addSection()
end

---@public
---@return number
function Toolbar:sectionsCount()
    local content = self:content()
    local sectionList = content:sectionList()
    local count = sectionList:count()
    return count
end
