import('lang.Object')
import('model.Cursor')

---@class Inventory : Object
---@field private _player LuaPlayer
Inventory = Object:extendAs("model.toolbar.section.table.Inventory")

---@public
---@return Inventory
---@param player_index number
function Inventory:new(player_index)
    local this = Inventory:super(Object:new())
    this._player = game.players[player_index]
    return this
end

---@public
---@param itemName string
---@return boolean
function Inventory:hasItem(itemName)
    return self:countOf(itemName) > 0
end

---@public
---@param itemName string
---@return number
function Inventory:countOf(itemName)
    local count = self:main().get_item_count(itemName)
    if self:hasTrash() then
        count = count + self:trash().get_item_count(itemName)
    end
    return count
end

---@public
---@param itemName string
function Inventory:pick(itemName)
    local stack, slotIndex, inventory = self:findStack(itemName)
    local successful = Cursor:new(self._player.index):pickStack(stack)
    if successful then
        self._player.hand_location = { inventory = inventory.index, slot = slotIndex }
    end
end

---@return LuaItemStack, number, LuaInventory
function Inventory:findStack(itemName)
    local stack, slotIndex, inventory
    if self:hasTrash() and self:trash().get_item_count(itemName) > 0 then
        stack, slotIndex = self:trash().find_item_stack(itemName)
        inventory = self:trash()
    else
        stack, slotIndex = self:main().find_item_stack(itemName)
        inventory = self:main()
    end
    return stack, slotIndex, inventory
end

---@private
---@return LuaInventory
function Inventory:main()
    return self._player.get_main_inventory()
end

---@private
---@return boolean
function Inventory:hasTrash()
    return self:trash() ~= nil
end

---@private
---@return LuaInventory
function Inventory:trash()
    return self._player.get_inventory(defines.inventory.character_trash)
end
