import('model.Component')
import('model.toolbar.section.table.Table')
import('model.toolbar.section.table.slots.Empty')
import('model.toolbar.section.table.slots.Item')
import('model.toolbar.section.table.slots.Tool')

---@class Row : Component
Row = Slot:extendAs("model.toolbar.section.table.Row")

function Row:create(parent)
    local root = self:createRoot(parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_content_sectionList_section_content_table_content_row" })
    local row = Row:new(root)
    local table = row:ancestor(Table)
    for _ = 1, table:columnsCount() ~= 0 and table:columnsCount() or 5 do
        Empty:create(root)
    end
    return row
end

---@public
---@return Row
---@param element LuaGuiElement
function Row:new(element)
    return Row:super(Component:new(element))
end

function Row:childrenClasses()
    return { Empty, Item, Tool }
end

function Row:addColumn()
    Empty:create(self:root())
end

---@return boolean
function Row:hasOccupiedLastSlot()
    return not Empty:is(self:lastSlotElement())
end

---@public
function Row:deleteLastSlot()
    self:lastSlotElement().destroy()
end

---@private
---@return LuaGuiElement
function Row:lastSlotElement()
    return self:root().children[self:slotsCount()]
end

---@private
---@return number
function Row:slotsCount()
    return #self:root().children
end

---@return boolean
function Row:hasOccupiedSlots()
    for _, slot in ipairs(self:root().children) do
        if not Empty:is(slot) then
            return true
        end
    end
    return false
end
